# pygame.mixer.music.load()  ——  载入一个音乐文件用于播放
# pygame.mixer.music.play()  ——  开始播放音乐流
# pygame.mixer.music.rewind()  ——  重新开始播放音乐
# pygame.mixer.music.stop()  ——  结束音乐播放
# pygame.mixer.music.pause()  ——  暂停音乐播放
# pygame.mixer.music.unpause()  ——  恢复音乐播放
# pygame.mixer.music.fadeout()  ——  淡出的效果结束音乐播放
# pygame.mixer.music.set_volume()  ——  设置音量
# pygame.mixer.music.get_volume()  ——  获取音量
# pygame.mixer.music.get_busy()  ——  检查是否正在播放音乐
# pygame.mixer.music.set_pos()  ——  设置播放的位置
# pygame.mixer.music.get_pos()  ——  获取播放的位置
# pygame.mixer.music.queue()  ——  将一个音乐文件放入队列中，并排在当前播放的音乐之后
# pygame.mixer.music.set_endevent()  ——  当播放结束时发出一个事件
# pygame.mixer.music.get_endevent()  ——  获取播放结束时发送的事件
import pygame
import time
import threading
import sys


class Audio:

    def __init__(self,
                 path: str,
                 if_track_pos=False
                 ):
        """
        音乐播放器的构造函数
        """
        pygame.mixer.init()
        self.if_track_pos = if_track_pos
        self.path = path
        self.track = None
        self._load()
        self.pos = -1

    def _load(self):
        """导入MP3文件"""
        self.track = pygame.mixer.music.load(self.path)

    def play(self):
        """开始播放"""
        pygame.mixer.music.play()

    def stop_all(self):
        """停止播放"""
        pygame.mixer.music.stop()

    def unpause(self):
        """停止暂停"""
        pygame.mixer.music.unpause()

    def pause(self):
        """暂停"""
        pygame.mixer.music.pause()

    def set_volume(self, value):
        """设置音量"""
        pygame.mixer.music.set_volume(value)

    def get_busy(self):
        """获取是否正在播放音频"""
        pygame.mixer.music.get_busy()

    def queue(self, next_filename):
        """设置下一首播放什么"""
        pygame.mixer.music.queue(next_filename)

    def start_at(self, position):
        """设置开始播放的位置"""
        if self.get_busy():
            self.stop_all()
        pygame.mixer.music.play(start=position)

    def get_sound_pos(self):
        """获取当前声音的进度"""
        if self.if_track_pos:
            self.pos = int(pygame.mixer.music.get_pos() / 1000)

    def set_sound_pos(self, pos):
        """设置开始播放的位置"""
        pygame.mixer.music.set_pos(pos)


if __name__ == "__main__":
    ad = Audio(path="../../sources/target.mp3", if_track_pos=True)
    ad.play()
    ad.set_sound_pos(0)
    time.sleep(10)
    ad.stop_all()
    sys.exit(0)



import copy
from abc import ABC

from SqrtS.tools.load_theme import Theme
from SqrtS.Widgets import Widget
from SqrtS.Widgets.Button import _SlideButton, BROADWISE
from SqrtS.Core.event_process import MOUSEUP, MOUSEDOWN, EVERYTIME
import pygame


class SlideBar(Widget.Widget):
    def __init__(self,
                 theme: Theme,
                 size: list,
                 button_size: list,
                 IDname: str, ):
        """
        不确定
        :param theme:
        :param size:
        :param button_size:
        :param IDname:

        """
        super().__init__(size, many_weights=False, IDname=IDname)
        self.move_out_function = None
        self.if_this_time_call_on = None
        self.move_on_function = None
        self.chosen = None
        self.up_function = None
        self.call_function = None
        self.pos = []
        # 导入素材     1.滑块正常背景  2.滑块被选择时的背景  3.滑块按钮正常时的样子  4.滑块按钮被选择时的样子  5.滑块按钮被点击拖动时的样子  6.背景覆盖的图像拉伸（推荐纯色）
        self.process_bar_base_img_formal = pygame.transform.scale(theme.get_theme()["sb_formal"], size)
        self.process_bar_chosen_img = pygame.transform.scale(theme.get_theme()["sb_chosen"], size)

        self.process_bar_head_button_img_formal = pygame.transform.scale(theme.get_theme()["sb_b_formal"], button_size)
        self.process_bar_head_button_img_chosen = pygame.transform.scale(theme.get_theme()["sb_b_chosen"], button_size)
        self.process_bar_head_button_img_click = pygame.transform.scale(theme.get_theme()["sb_b_click"], button_size)

        self.now_bar_img = self.process_bar_base_img_formal
        self.now_button_img = self.process_bar_head_button_img_formal

        # button_size,img_list,motion_range,limit_direction
        self.slide_button = _SlideButton([self.size[1], self.size[1]],
                                         [self.process_bar_head_button_img_formal,
                                          self.process_bar_head_button_img_click,
                                          self.process_bar_head_button_img_chosen],
                                         BROADWISE, IDname="process_bar_button")

        self.event_types = [MOUSEDOWN, MOUSEUP, EVERYTIME]

    def bind_pos(self, new_pos: list):
        """
        绑定新坐标
        """
        self.pos = new_pos
        pos_copy = copy.copy(new_pos)
        self.slide_button.bind_pos(pos_copy)
        self.slide_button.bind_move_length(pos_copy, self.size[0])

    def blit(self, window):
        """
        在屏幕上画出图
        """
        window.get_window().blit(self.now_bar_img, self.pos)
        self.slide_button.blit(window)

    def _compute_value(self):
        """计算值"""
        ...

    @staticmethod
    def _compute_central_pos(left_top, size):
        return [int(left_top[0] + size[0] / 2), int(left_top[1] + size[1])]

    def check_mouse_down(self, event, window):
        """
        检测鼠标按下并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_click_pos[
            1] < \
                self.pos[1] + self.size[1]:
            self.locate_pos_by_mouse(mouse_click_pos)
            if self.call_function:
                self.call_function(self)
        else:
            ...

    def check_mouse_up(self, event, window):
        """
        检测鼠标抬起并且调用回调函数的函数（笑）
        :param event:事件
        :return:None
        """
        mouse_click_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_click_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_click_pos[
            1] < \
                self.pos[1] + self.size[1]:
            if self.up_function:
                self.up_function(self)
        else:
            ...

    def weight_everytime_event(self, window):
        """
        每时每刻都在运行的函数为everytime函数，按钮内用于处理按钮移动上去的事件
        :return: None
        """
        mouse_pos = pygame.mouse.get_pos()
        if self.pos[0] < mouse_pos[0] < self.pos[0] + self.size[0] and self.pos[1] < mouse_pos[1] < \
                self.pos[1] + self.size[1]:
            self.now_bar_img = self.process_bar_chosen_img
            if self.move_on_function and not self.if_this_time_call_on:
                self.move_on_function(self)
                self.if_this_time_call_on = True
        elif self.chosen:
            self.now_bar_img = self.process_bar_base_img_formal
            if self.move_out_function:
                self.move_out_function(self)
        else:
            self.now_bar_img = self.process_bar_base_img_formal

    def register_for_event(self, window):
        window.event_processor.register_event(MOUSEDOWN, self.check_mouse_down)
        window.event_processor.register_event(MOUSEUP, self.check_mouse_up)
        window.event_processor.register_event(EVERYTIME, self.weight_everytime_event)
        self.slide_button.register_for_event(window=window)

    def bind_call_func(self, func):
        self.call_function = func

    def bind_up_func(self, func):
        self.up_function = func

    def bind_on_func(self, func):
        self.move_on_function = func

    def bind_out_func(self, func):
        self.move_out_function = func

    def locate_pos_by_mouse(self, mouse_pos):
        self.slide_button.bind_pos([mouse_pos[0] - int(self.size[1] / 2), self.slide_button.pos[1]])

    def get_value(self):
        return self._compute_value()


class FollowSlideBar(SlideBar):
    def __init__(self, theme: Theme,
                 size: list,
                 button_size: list,
                 IDname: str):
        super().__init__(theme, size, button_size, IDname)

        self.process_bar_over_formal = pygame.transform.scale(theme.get_theme()["sb_cover"], size)

    def update_over_size(self):
        self.process_bar_over_formal = pygame.transform.scale(self.process_bar_over_formal,
                                                              [self.slide_button.pos[0], self.size[1]])

    def blit(self, window):
        self.update_over_size()
        window.get_window().blit(self.now_bar_img, self.pos)
        window.get_window().blit(self.process_bar_over_formal, self.pos)
        self.slide_button.blit(window)


class ProcessBar(Widget.Widget):
    def __init__(self, theme: Theme,
                 size: list,
                 IDname: str,
                 unit_length: int):
        """
        进度条
        :param theme: Theme
        :param size: 单位长度下的尺寸
        :param IDname: ID
        :param unit_length:单位长度
        例如，size为【1,100】，unit_length=2，意为尺寸为1-100，每一个尺寸数值对应像素长度2
        """
        super().__init__(size, IDname)

        self.unit_size = unit_length
        self.size = [size[0] * unit_length, size[1] * unit_length]
        self.IDname = IDname

        self.max_length = int(self.size[0] / self.unit_size)

        self.over_size = 1
        self.pos = []

        self.bar_formal_img = pygame.transform.scale(theme.get_theme()["pb_formal"], self.size)
        self.bar_over_img = theme.get_theme()["pb_over"]

    def bind_pos(self, new_pos: list):
        self.pos = new_pos

    def update_over(self):
        self.bar_over_img = pygame.transform.scale(self.bar_over_img, [self.over_size * self.unit_size, self.size[1]])

    def blit(self, window):
        self.update_over()
        window.get_window().blit(self.bar_formal_img, self.pos)
        window.get_window().blit(self.bar_over_img, self.pos)

    def register_for_event(self, window):
        ...

    def add_process(self, num):
        if (self.over_size + num) <= self.max_length:
            self.over_size += num
        else:
            self.over_size = self.max_length

    def cut_process(self, num):
        if (self.over_size - num) >= 0:
            self.over_size -= num
        else:
            self.over_size = 0

    def set_process(self, num):
        if 0 <= num <= self.max_length:
            self.over_size = num


if __name__ == "__main__":
    ...



import os
import pyautogui
import pygame
from SqrtS.Expand.WindowTransparency import window_transparency
from SqrtS.Errors.core_error import WidgetNotFound
from SqrtS.Expand.WindowBorder import WindowBorder
from SqrtS.Expand.DraggingBoard import GlobalDraggingBoard
from SqrtS.Core.event_process import EventProcessor
from SqrtS.Core.paging import Pager
from SqrtS.Core.AnimationSystem import AnimationSystem
from SqrtS.tools.image import ImageSurface
from SqrtS.Expand.WindowMove import WindowMover
from SqrtS.Core.TaskSystem import TaskSystem, ONCE, INFINITE, TIMES, TERMINABLE_INFINITE
import win32api

LEFT = 1
MID = 2
RIGHT = 3
WHEEL_DOWN = 5
WHEEL_UP = 4


def get_screen_pixel():
    screen_width = win32api.GetSystemMetrics(0)
    screen_height = win32api.GetSystemMetrics(1)
    return [screen_width, screen_height]


def os_close():
    os._exit(0)


class Window:
    def __init__(self,
                 window_size: tuple,
                 fps=100,
                 noframe=False,
                 _no_screen=False
                 ):
        """
        窗口类的构造函数
        :param window_size:窗口大小，传入元组（int，int）
        :param fps:帧率
        :param noframe:是否有边框？
        :param _no_screen:不用管！如果为True，证明是用于提示框的窗口，不具备自己的屏幕
        """
        os.environ["SDL_IME_SHOW_UI"] = "1"  # 显示输入候选框UI
        # 窗口大小
        self.window_size = window_size
        self._no_screen = _no_screen
        self.noframe = noframe
        # 设置窗口大小
        if not _no_screen:
            if not noframe:
                self.screen = pygame.display.set_mode(self.window_size)
            else:
                self.screen = pygame.display.set_mode(self.window_size, pygame.NOFRAME)
        # 设置窗存在状态
        self.alive = True

        # 窗口布局文件
        self.Layout = None

        # FPS
        self.fps = fps

        # 定义FPS调控时钟
        self.clock = pygame.time.Clock()

        # 定义事件管理器
        self.event_processor = EventProcessor()

        # 定义任务系统
        self.task_system = TaskSystem()

        # 定义分页器
        self.pager = Pager()

        # background_image
        self.background_image = None

        self.background_color = (25, 25, 25)

        self.title = "SqrtS GUI window"
        pygame.display.set_caption(self.title)

        # 动画刷新系统
        self.animation_system = AnimationSystem()

        # 是否正在暂停？
        self.is_pausing = False

    def __str__(self):
        """
        魔术方法，返回对象的基本信息属性
        :return:
        """
        return f"[对象基本信息]：<SqrtS.Core.Window>=[alive:{self.alive},window_size{self.window_size}]"

    def bind_background_color(self, new_color):
        """
        绑定背景颜色（其实内部也会调用，设置窗口透明就是用这种方法）
        :param new_color: (int,int,int)
        :return: None
        """
        self.background_color = new_color

    def get_widget_by_IDname(self, IDname):
        """
        通控件的IDname获取控件
        :param IDname: str
        :return: None
        """
        for i in self.Layout.widgets:
            if i.IDname == IDname:
                return i
        raise WidgetNotFound(f"没有IDname为{IDname}的控件！")

    def set_window_name(self, name):
        """
        设置窗口名称
        :param name: str
        :return: None
        """
        self.title = name
        pygame.display.set_caption(name)

    @staticmethod
    def set_window_icon(icon_path):
        """
        设置窗口图标
        :param icon_path:图片路径
        :return: None
        """
        icon_surface = pygame.image.load(icon_path)
        pygame.display.set_icon(icon_surface)

    def bind_background_image(self, background_surface: ImageSurface, resize=False | True):
        """
        设置背景图片
        :param background_surface: 背景图片！ImageSurface类型！
        :param resize: 要么False，表示不用重新调整大小；要么True,表示背景图片全部显示，会被重设大小
        :return: None
        """
        if not resize:
            self.background_image = background_surface.return_surface()
        else:
            self.background_image = background_surface.resize(self.get_window_size())

    def get_window_size(self):
        """
        获得窗口大小（并没有什么用的方法，可能是用来凑数的）
        :return: 窗口大小元组
        """
        return self.window_size

    def get_window(self):
        """
        获得窗口
        :return: pygame.Screen
        """
        return self.screen

    def bind_layout(self, layout):
        """
        绑定当前窗口的布局类，可以重复绑定更新
        :param layout: 布局类
        :return: None
        """
        self.Layout = layout

    def register_for_animation(self, animation):
        """

        :param animation: 动画对象
        :return: None
        """
        self.animation_system.register_for_animation(animation)

    def register_for_dynamic_effect(self, animation):
        """

        :param animation: 动画对象
        :return: None
        """
        self.animation_system.register_for_dynamic_effects(animation)

    def destroy_animation(self, animation):
        """

        :param animation: 动画对象
        :return: None
        """
        self.animation_system.destroy_animation(animation)

    def destroy_dynamic_effect(self, animation):
        """

        :param animation: 动画对象
        :return: None
        """
        self.animation_system.destroy_dynamic_effects(animation)

    def register_for_task(self, type_, func, time_interval, times=None, args=None, task_name=None):
        """
        注册任务
        :param task_name: 在注册可停止的无限任务时需要你用到
        :param args: 参数，在注册多次任务时需要用到
        :param type_: 类型，可选地有INFINITE(永久任务，虽然我不知道为什么取无限这个明细),ONCE(只运行一次的),TIMES(多次的，自己指定次数)
        :param func: 目标任务函数
        :param time_interval: 时间间隔
        :param times: 次数，仅当你选择TIMES为type_时的必填参数
        :return:None
        """
        if type_ == INFINITE:
            self.task_system.register_for_infinite_task(task_func=func, time_interval=time_interval)
        elif type_ == TIMES:
            self.task_system.register_for_times_task(task_func=func, times=times, time_interval=time_interval,
                                                     args=args)
        elif type_ == ONCE:
            self.task_system.register_fot_once_task(task_func=func, time_interval=time_interval, args=args)
        elif type_ == TERMINABLE_INFINITE:
            self.task_system.register_for_terminable_infinite_task(task_func=func, time_interval=time_interval,
                                                                   task_name=task_name)

    def destroy_terminable_task(self, task_name):
        self.task_system.destroy_terminable_infinite_task(task_name)

    def append_page(self, layout):
        """
        向窗口的分页器添加一个页面
        :param layout: 布局类
        :return: NOne
        """
        self.pager.append_page(layout, self)

    def change_page(self, layout_name, reload_function):
        """
        改变一个页面
        :param reload_function: 注册函数，需要在新的事件管理中注册事件
        :param layout_name:str
        :return: None
        """
        self.pager.change_page(self, layout_name, reload_function)

    def _reload_layout(self, new_layout, reload_function):
        """
        更换页面，实现切换页面的功能~
        :param reload_function: 注册函数，需要在新的事件管理中注册事件
        :param new_layout: 新的布局文件
        :return: 你猜一猜有没有？
        """
        self.event_processor = EventProcessor()
        self.Layout = new_layout
        reload_function()

    def close(self):
        """
        关闭窗口
        :return:
        """
        self.alive = False
        os_close()
        exit()

    def run(self):
        """
        主运行函数，所有的操作和刷新最终都汇聚到这个方法
        :return: 你猜有没有？
        """
        pygame.init()
        while self.alive:

            # 控制FPS
            self.screen.fill(self.background_color)
            self.animation_system.update_blit_animations(self)
            if self.background_image:
                # 如果有背景图片，则绘制
                self.screen.blit(self.background_image, (0, 0))
            self.clock.tick(self.fps)
            # 布局的全局绘图功能调用
            if self.Layout:
                self.Layout.blit_all(self)
            try:
                # 循环检测事件
                for event in pygame.event.get():
                    if event.type == pygame.QUIT:
                        os_close()
                        exit()
                    elif event.type == pygame.KEYDOWN:
                        self.event_processor.key_down_process(event, self)
                    elif event.type == pygame.KEYUP:
                        self.event_processor.key_up_process(event, self)
                    elif event.type == pygame.MOUSEBUTTONDOWN:

                        if event.button == LEFT:
                            self.event_processor.mouse_left_down(event, self)
                        elif event.button == RIGHT:
                            self.event_processor.mouse_right_down(event, self)
                        elif event.button == MID:
                            self.event_processor.mouse_mid_down(event, self)
                        elif event.button == WHEEL_DOWN:
                            self.event_processor.mouse_wheel_down(event, self)
                        elif event.button == WHEEL_UP:
                            self.event_processor.mouse_wheel_up(event, self)

                        self.event_processor.mouse_down(event, self)

                    elif event.type == pygame.MOUSEBUTTONUP:

                        if event.button == LEFT:
                            self.event_processor.mouse_left_up(event, self)
                        elif event.button == RIGHT:
                            self.event_processor.mouse_right_up(event, self)
                        elif event.button == MID:
                            self.event_processor.mouse_mid_up(event, self)

                        self.event_processor.mouse_up(event, self)

                    elif event.type == pygame.DROPFILE:
                        self.event_processor.drop_file_event(event, self)
                    elif event.type == pygame.TEXTINPUT:
                        self.event_processor.text_input_event(event, self)
                self.event_processor.everytime_event(self)
                self.task_system.task_system_update()
            except pygame.error as e:
                continue
            except ValueError as e:
                continue
            # 画图更新
            pygame.display.flip()

    @staticmethod
    def iconify():
        pygame.display.iconify()


class InferiorWindow(Window):
    def __init__(self, window):
        super().__init__(window_size=window.get_window_size(), fps=50, _no_screen=True)
        self.screen = window.screen

        # {"name":[surface,pos]}
        self.draw_things = {}

    def bind_background_image(self, background_surface: ImageSurface, resize=False | True):
        """
        设置背景图片(重写为空函数)
        """
        ...

    def append_draw_thing(self, new_thing):
        """
        添加需要被画出的新东西
        :param new_thing: [name, surface, pos]
        :return: None
        """
        self.draw_things[new_thing[0]] = new_thing[1:]

    def run(self):
        """
        主运行函数，所有的操作和刷新最终都汇聚到这个方法
        :return: 你猜有没有？
        """
        while self.alive:
            # 控制FPS
            if self.draw_things:
                for i in self.draw_things.keys():
                    self.screen.blit(self.draw_things[i][0], self.draw_things[i][1])
            self.animation_system.update_blit_animations(self)

            self.clock.tick(self.fps)
            # 布局的全局绘图功能调用
            self.Layout.blit_all(self)

            # 循环检测事件
            try:
                for event in pygame.event.get():
                    if event.type == pygame.QUIT:
                        os_close()
                        exit()
                    elif event.type == pygame.KEYDOWN:
                        self.event_processor.key_down_process(event, self)
                    elif event.type == pygame.KEYUP:
                        self.event_processor.key_up_process(event, self)
                    elif event.type == pygame.MOUSEBUTTONDOWN:

                        if event.button == LEFT:
                            self.event_processor.mouse_left_down(event, self)
                        elif event.button == RIGHT:
                            self.event_processor.mouse_right_down(event, self)
                        elif event.button == MID:
                            self.event_processor.mouse_mid_down(event, self)
                        elif event.button == WHEEL_DOWN:
                            self.event_processor.mouse_wheel_down(event, self)
                        elif event.button == WHEEL_UP:
                            self.event_processor.mouse_wheel_up(event, self)

                        self.event_processor.mouse_down(event, self)

                    elif event.type == pygame.MOUSEBUTTONUP:

                        if event.button == LEFT:
                            self.event_processor.mouse_left_up(event, self)
                        elif event.button == RIGHT:
                            self.event_processor.mouse_right_up(event, self)
                        elif event.button == MID:
                            self.event_processor.mouse_mid_up(event, self)

                        self.event_processor.mouse_up(event, self)

                    elif event.type == pygame.DROPFILE:
                        self.event_processor.drop_file_event(event, self)
                    elif event.type == pygame.TEXTINPUT:
                        self.event_processor.text_input_event(event, self)
                self.event_processor.everytime_event(self)
                self.task_system.task_system_update()
            except pygame.error:
                continue
            except ValueError:
                continue
            # 画图更新
            pygame.display.flip()


class CustomizedFrameWindow(Window):
    """
    自定义窗口边框的窗口
    """

    def __init__(self, window_size: tuple,
                 window_frame_size: int,
                 theme,
                 font,
                 enable_customized_border=False,
                 customised_border=None):
        super().__init__(window_size, noframe=True)
        self.real_window_size = [window_size[0], window_size[1] + window_frame_size]
        self.window_size = window_size
        self.screen = pygame.display.set_mode([window_size[0], window_size[1] + window_frame_size], pygame.NOFRAME)
        self.offset_address = window_frame_size

        if not enable_customized_border:
            self.window_border = WindowBorder(theme, border_size=[window_size[0], window_frame_size], window=self,
                                              font=font)
        else:
            if customised_border:
                self.window_border = customised_border

        self.dragging_board = GlobalDraggingBoard([self.window_size[0], self.offset_address], "_slide_board")
        self.dragging_board.register_for_event(self)
        self.dragging_board.bind_pos([0, 0])
        self.dragging_board.bind_call_slide(self._drag)

        self.window_mover = WindowMover()

    def _drag(self):
        """
        拖拽函数，内部函数
        :return: None
        """
        abs_mouse_pos = pyautogui.position()
        pygame_pos = self.dragging_board.last_mouse_pos
        self.window_mover.moveto([abs_mouse_pos[0] - pygame_pos[0], abs_mouse_pos[1] - pygame_pos[1]])

    def get_real_window_size(self):
        """
        获得窗口大小（并没有什么用的方法，可能是用来凑数的）
        :return: 窗口大小元组
        """
        return self.real_window_size

    def get_frame_offset(self):
        """
        获取边框的高度
        :return: 高度int
        """
        return self.offset_address

    def bind_layout(self, layout):
        """
        绑定当前窗口的布局类，可以重复绑定更新
        :param layout: 布局类
        :return: None
        """

        for i in layout.widgets:
            i.bind_pos([i.pos[0], i.pos[1] + self.offset_address])

        self.Layout = layout

    def append_page(self, layout):
        """
        向窗口的分页器添加一个页面
        :param layout: 布局类
        :return: NOne
        """
        self.pager.append_page(layout, self)

    def run(self):
        self.window_mover.update(self)
        self.dragging_board.update_info(self)
        """
        主运行函数，所有的操作和刷新最终都汇聚到这个方法
        :return: 你猜有没有？
        """
        while self.alive:
            # 控制FPS
            self.screen.fill(self.background_color)
            self.window_border.blit_border(self)
            if self.background_image:
                self.screen.blit(self.background_image, (0, self.offset_address))
            self.clock.tick(self.fps)
            # 布局的全局绘图功能调用
            self.Layout.blit_all(self)
            # 循环检测事件
            try:
                for event in pygame.event.get():
                    if event.type == pygame.QUIT:
                        os_close()
                        exit()
                    elif event.type == pygame.KEYDOWN:
                        self.event_processor.key_down_process(event, self)
                    elif event.type == pygame.KEYUP:
                        self.event_processor.key_up_process(event, self)
                    elif event.type == pygame.MOUSEBUTTONDOWN:

                        if event.button == LEFT:
                            self.event_processor.mouse_left_down(event, self)
                        elif event.button == RIGHT:
                            self.event_processor.mouse_right_down(event, self)
                        elif event.button == MID:
                            self.event_processor.mouse_mid_down(event, self)
                        elif event.button == WHEEL_DOWN:
                            self.event_processor.mouse_wheel_down(event, self)
                        elif event.button == WHEEL_UP:
                            self.event_processor.mouse_wheel_up(event, self)

                        self.event_processor.mouse_down(event, self)

                    elif event.type == pygame.MOUSEBUTTONUP:

                        if event.button == LEFT:
                            self.event_processor.mouse_left_up(event, self)
                        elif event.button == RIGHT:
                            self.event_processor.mouse_right_up(event, self)
                        elif event.button == MID:
                            self.event_processor.mouse_mid_up(event, self)

                        self.event_processor.mouse_up(event, self)

                    elif event.type == pygame.DROPFILE:
                        self.event_processor.drop_file_event(event, self)
                    elif event.type == pygame.TEXTINPUT:
                        self.event_processor.text_input_event(event, self)
                self.event_processor.everytime_event(self)
                self.task_system.task_system_update()
            except pygame.error:
                continue
            except ValueError:
                continue
            # 画图更新
            pygame.display.flip()
        if self.is_pausing:
            self.pausing_run()
            if self.alive:
                self.run()


class FullScreenWindow(Window):
    def __init__(self, if_transparency=False):
        super().__init__((500, 500))
        self.window_size = get_screen_pixel()
        self.screen = pygame.display.set_mode(self.window_size, pygame.NOFRAME | pygame.FULLSCREEN)
        self.if_transparency = if_transparency

    def run(self):
        """
        主运行函数，所有的操作和刷新最终都汇聚到这个方法
        :return: 你猜有没有？
        """
        if self.if_transparency:
            window_transparency(self)
        while self.alive:
            # 控制FPS
            self.screen.fill(self.background_color)
            if self.background_image:
                # 如果有背景图片，则绘制
                self.screen.blit(self.background_image, (0, 0))
            self.clock.tick(self.fps)
            self.animation_system.update_blit_animations(self)
            # 布局的全局绘图功能调用
            self.Layout.blit_all(self)
            # 循环检测事件
            try:
                for event in pygame.event.get():
                    if event.type == pygame.QUIT:
                        os_close()
                        exit()
                    elif event.type == pygame.KEYDOWN:
                        self.event_processor.key_down_process(event, self)
                    elif event.type == pygame.KEYUP:
                        self.event_processor.key_up_process(event, self)
                    elif event.type == pygame.MOUSEBUTTONDOWN:

                        if event.button == LEFT:
                            self.event_processor.mouse_left_down(event, self)
                        elif event.button == RIGHT:
                            self.event_processor.mouse_right_down(event, self)
                        elif event.button == MID:
                            self.event_processor.mouse_mid_down(event, self)
                        elif event.button == WHEEL_DOWN:
                            self.event_processor.mouse_wheel_down(event, self)
                        elif event.button == WHEEL_UP:
                            self.event_processor.mouse_wheel_up(event, self)

                        self.event_processor.mouse_down(event, self)

                    elif event.type == pygame.MOUSEBUTTONUP:

                        if event.button == LEFT:
                            self.event_processor.mouse_left_up(event, self)
                        elif event.button == RIGHT:
                            self.event_processor.mouse_right_up(event, self)
                        elif event.button == MID:
                            self.event_processor.mouse_mid_up(event, self)

                        self.event_processor.mouse_up(event, self)

                    elif event.type == pygame.DROPFILE:
                        self.event_processor.drop_file_event(event, self)
                    elif event.type == pygame.TEXTINPUT:
                        self.event_processor.text_input_event(event, self)
                self.event_processor.everytime_event(self)
                self.task_system.task_system_update()
            except pygame.error:
                continue
            except ValueError:
                continue
            # 画图更新
            pygame.display.flip()


if __name__ == "__main__":
    print(get_screen_pixel())



import os
from abc import abstractmethod
from SqrtS.Core.TaskSystem import TIMES
import pygame.image
from SqrtS.tools.load_theme import unzip_file, walk_one
from SqrtS.tools._json_control import JsonReader
from SqrtS.Errors.animation_err import NotAnimationFileError, UnpackSqsaError


class Animation:
    """
    动画基础类
    """

    def __init__(self, IDname):
        self.had_been_play = False
        self.IDname = IDname
        self.surfaces_dict = {}
        self.animation_path = ""
        self.animation_name = ""
        self.setting = {}

        self.img_index = 0
        self.now_img = None

        self.pos = []

        self.alive = True

    def times_blit_task(self):
        if self.img_index + 1 < len(self.surfaces_dict):
            self.img_index += 1
            self.now_img = self.surfaces_dict[str(self.img_index)]
        else:
            self.alive = False

    @abstractmethod
    def play(self, window, pos):
        self.pos = pos
        self.now_img = self.surfaces_dict[str(self.img_index)]
        if not self.had_been_play:
            window.register_for_task(TIMES,
                                     self.times_blit_task,
                                     time_interval=self.setting["speed"], times=len(self.surfaces_dict))
            window.register_for_animation(self)
        self.had_been_play = True

    def blit_flip(self, window):
        if self.alive:
            window.get_window().blit(self.now_img, self.pos)
        else:
            window.destroy_animation(animation=self)

    def load_animation(self, path):
        if "\\" in path:
            type_ = "\\"
        else:
            type_ = "/"

        if path.split(".")[-1] == "sqsa":
            new_path = "".join(path.split(".")[:-1]) + ".zip"
            os.rename(path, new_path)
            self.animation_path = path
            self.animation_name = self.animation_path.split(type_)[-1].split(".")[0]
        else:
            raise NotAnimationFileError("您传入的路径指向的并不是一个sqsa帧动画文件！")

        target_path = ""
        for i in path.split(type_)[0:-1]:
            target_path = target_path + i + type_

        result = unzip_file(new_path, target_path)
        os.rename(new_path, path)

        if not result:
            raise UnpackSqsaError("您传入的sqsa帧动画文件可能已经损坏或者路径错误！解压失败！")
        else:
            setting_path = "".join(path.split(".")[:-1]) + type_ + "setting.json"
            self.setting = JsonReader(json_path=setting_path).read_json()
            walk_path = "".join(path.split(".")[:-1]) + type_
            for filename in walk_one(walk_path):
                if filename.split(".")[-1] in ["png", "PNG", "jpg", "JPG", "JPEG", "jpeg"]:
                    flip_name = filename.split(".")[0]
                    self.surfaces_dict[flip_name] = pygame.image.load(walk_path + filename)


class LowerAnimation:
    """
    动画基础类，但是是更加底层的
    """

    def __init__(self, IDname):
        self.had_been_play = False
        self.IDname = IDname
        self.surfaces_dict = {}
        self.animation_path = ""
        self.animation_name = ""
        self.setting = {}

        self.img_index = 0
        self.now_img = None

        self.pos = []

        self.alive = True

        self.style = "\\"

    def times_blit_task(self):
        if self.img_index + 1 < len(self.surfaces_dict):
            self.img_index += 1
            self.now_img = self.surfaces_dict[str(self.img_index)]
        else:
            self.alive = False

    @abstractmethod
    def play(self, window, pos, speed):
        self.pos = pos
        self.now_img = self.surfaces_dict[str(self.img_index)]
        if not self.had_been_play:
            window.register_for_task(TIMES,
                                     self.times_blit_task,
                                     time_interval=speed, times=len(self.surfaces_dict))
            window.register_for_animation(self)
        self.had_been_play = True

    def blit_flip(self, window):
        if self.alive:
            window.get_window().blit(self.now_img, self.pos)
        else:
            window.destroy_animation(animation=self)

    def load_animation(self, path, size=None):
        if "/" in path:
            self.style = "/"
        if size:
            for filename in walk_one(path):
                if filename.split(".")[-1] in ["png", "PNG", "jpg", "JPG", "JPEG", "jpeg"]:
                    flip_name = filename.split(".")[0]
                    self.surfaces_dict[flip_name] = pygame.transform.scale(
                        pygame.image.load(path + self.style + filename), size)
        else:
            for filename in walk_one(path):
                if filename.split(".")[-1] in ["png", "PNG", "jpg", "JPG", "JPEG", "jpeg"]:
                    flip_name = filename.split(".")[0]
                    self.surfaces_dict[flip_name] = pygame.image.load(path + self.style + filename)


if __name__ == "__main__":
    ...



import pygame
import os
from SqrtS.Widgets.Button import ImageButton
from SqrtS.Widgets.Text import Texts


class FalseWindow:
    """
    闲的没事写的，忘记删了
    """
    def __init__(self, true_window):
        self.screen = true_window.get_window()

    def get_window(self):
        """
        获取窗口
        :return:
        """
        return self.screen


class WindowBorder:
    """
    基础窗口框
    """
    def __init__(self, theme,
                 border_size: list,
                 window,
                 font,
                 icon_path=None,
                 icon_size=(25, 25),
                 title="SqrtS GUI",
                 title_color=(255, 255, 255),
                 title_background=None,
                 button_length=30,
                 button_interval=3, ):
        """
        参数
        :param theme: Theme
        :param border_size:[int,int]
        :param window: Window
        :param font: SqrtSFont
        :param icon_path: str
        :param icon_size: [int,int]
        :param title: str
        :param title_color:[int,int,int]
        :param title_background: [int,int,int]
        :param button_length: int
        :param button_interval: int
        """
        self.pos = (0, 0)

        self.title_color = title_color
        self.title_background = title_background
        self.icon_size = icon_size

        self.font = font
        self.title = title
        self.title_sur = self.font.render_text(text=self.title, color=title_color, background=title_background)
        if not icon_path:
            self.icon_path = pygame.transform.scale(theme.get_theme()["formal_icon"], icon_size)
        else:
            self.icon_path = pygame.transform.scale(pygame.image.load(icon_path), icon_size)

        self.background = pygame.transform.scale(theme.get_theme()["border"], border_size)
        self.close_btn_img_list = [
            theme.get_theme()["kill_window"],
            theme.get_theme()["kill_window_click"],
            theme.get_theme()["kill_window_chosen"]
        ]
        self.cancel_btn_img_list = [
            theme.get_theme()["cancel"],
            theme.get_theme()["cancel_click"],
            theme.get_theme()["cancel_chosen"]
        ]
        self.close_btn = ImageButton(button_size=[button_length, border_size[1]], image_list=self.close_btn_img_list,
                                     IDname="close")
        self.min_button = ImageButton(button_size=[button_length, border_size[1]], image_list=self.cancel_btn_img_list,
                                      IDname="cancel")
        self.close_btn.bind_call_func(self._close)
        self.min_button.bind_call_func(self._iconify_window)

        self.close_btn.bind_pos([window.get_window_size()[0] - self.close_btn.size[0], 0])
        self.min_button.bind_pos(
            [window.get_window_size()[0] - self.close_btn.size[0] - button_interval - self.min_button.size[0], 0])

        self.close_btn.register_for_event(window)
        self.min_button.register_for_event(window)

    @staticmethod
    def _iconify_window(args):
        """
        最小化窗口的函数
        :param args: 固定，无实意
        :return: None
        """
        pygame.display.iconify()

    @staticmethod
    def _close(args):
        """
        内置关闭函数
        :param args:无实意
        :return: None
        """
        os._exit(0)

    def set_window_title(self, title):
        """
        设置窗口标题
        :param title: 标题str
        :return: None
        """
        self.title = title
        self.title_sur = self.font.render_text(text=self.title, color=self.title_color,
                                               background=self.title_background)

    def blit_border(self, window):
        """
        画出窗口框
        :param window:Window
        :return: None
        """
        window.get_window().blit(self.background, self.pos)
        screen = window.get_window()
        self.close_btn.blit(window)
        self.min_button.blit(window)
        self._blit_title(screen)
        self._blit_icon(screen)

    def _blit_title(self, screen):
        """
        内部函数，画出题目
        :param screen: pygame.surface
        :return: None
        """
        screen.blit(self.title_sur, (self.icon_size[0] + 3, 3))

    def _blit_icon(self, screen):
        """
        内部函数，画出图标图片
        :param screen: pygame.surface
        :return: None
        """
        screen.blit(self.icon_path, (1, 1))


class CustomizedWindowBorder:
    """
    自定义窗口边框
    """
    def __init__(self, theme,
                 border_size: list,
                 if_transparency=False):
        """
        参数
        :param theme:Theme
        :param border_size: [int,int]
        :param if_transparency: bool
        """
        self.pos = (0, 0)
        self.border_size = border_size

        self.background = pygame.transform.scale(theme.get_theme()["border"], self.border_size)

        self.widgets = []

        self.layout = None

        self.if_transparency = if_transparency

    def set_border_img(self, path):
        """
        设置边框的图片
        :param path: 图片路径
        :return: None
        """
        self.background = pygame.transform.scale(pygame.image.load(path), self.border_size)

    def bind_layout(self, layout):
        """
        绑定布局
        :param layout: Layout子类
        :return: None
        """
        self.layout = layout

    @staticmethod
    def _iconify_window(args):
        """
        最小化窗口
        :param args:固定
        :return: None
        """
        pygame.display.iconify()

    @staticmethod
    def _close(args):
        """
        内置关闭函数
        :param args:固定
        :return: None
        """
        os._exit(0)

    def blit_border(self, window):
        """
        画出窗口边框
        :param window:Window
        :return: None
        """
        if not self.if_transparency:
            window.get_window().blit(self.background, self.pos)
        else:
            window.get_window().fill(window.background_color)
        self.layout.blit_all(window)
